from django.shortcuts import render, get_object_or_404, get_list_or_404, redirect, HttpResponseRedirect
from django.contrib.auth.models import User, Group
from django.contrib import messages
from django.db.models import Q
from django.core.paginator import Paginator
from django.contrib.auth.decorators import login_required, permission_required

from django.contrib.auth import login, authenticate  # add to imports
import logging
from .models import Subscriber,Enquiry,FollowUp
logger = logging.getLogger(__name__)


def login_page(request):
    message = ''
    if request.method == 'POST':
        if User.objects.filter(username=request.POST['username']).exists():
            userobj= User.objects.get(username=request.POST['username'])
            if userobj.is_active == False:
                messages.error(request, "User is Inactive . Please contact to administrator")
                
            user = authenticate(
                username=request.POST['username'],
                password=request.POST['password'],
            )
            if user is not None:
                login(request, user)
                message = f'Hello {user.username}! You have been logged in'
                return HttpResponseRedirect('/dash')
                
            
            else:
                messages.error(request, "incorrect username and password")
        else:
            messages.error(request, "User does not exists.")
        
    return render(
        request, 'registration/login.html', context={'message': message})
    
def createEnq(request):
    Enquiry.objects.create(
    enqName=request.POST['name'],
    enqEmail=request.POST['email'],
    enqContact=request.POST['contact'],
    enqCompany=request.POST['company'],
    enqMessage=request.POST['msg'],
    enqProduct=request.POST['product']
    )
    return HttpResponseRedirect('/thanks/')


def followup_assign(request, enq):
    FollowUp.objects.create(
        assignTo=request.POST['assignTo'], remarks=request.POST['remark'], enquiry=enq)
    enq.enqStatus = request.POST['status']
    enq.save()


def subscribe(request):
    if Subscriber.objects.filter(email=request.POST['email']).exists():
        message='Your are already subscribed !'
    else:
        Subscriber.objects.create(email=request.POST['email'])
        userObj=User.objects.create(username=request.POST['email'],email=request.POST['email'])
        userObj.set_password(request.POST['email'])
        userObj.save()
        message='Thanks for Subscribe <br/> now you can login by your email'
    return render(request,'Pages/thanks.html',{'message':message})
    # return HttpResponseRedirect("/")

def signup(request):
    if request.method == 'POST':
        if User.objects.filter(username=request.POST['email']).exists():
            messages.success(request,"User  "+request.POST['email']+" already exists")
            return render(request, 'registration/signup.html')

        if request.POST['password1'] != request.POST['password2']:
            messages.success(request,"Password & Re-Type Password did't match, Try again.")
            return render(request, 'registration/signup.html')
            
        else:
            userObj = User.objects.create(
                username=request.POST['email'],
                first_name=request.POST['fName'],
                email=request.POST['email'],
                last_name=request.POST['lName'],
                is_staff=False,
                is_active=True
            )
            userObj.set_password(request.POST['password1'])
            userObj.save()
            messages.success(request, "User Created Successfully")
            return HttpResponseRedirect('/thanks/')
        return render(request, 'registration/signup.html')
    else:
        return render(request, 'registration/signup.html')


@login_required
def book_blog_list(request):
    try:
        per_page = int(request.GET['q'])
    except Exception as e:
        per_page = 10     # default value
    blog_list = request.user.bookmark_blogs.all()
    paginator = Paginator(blog_list, per_page)
    page_number = request.GET.get("page")
    posts = paginator.get_page(page_number)
    return render(request, 'Dashboard/blogs.html', {'posts': posts})


@login_required
def book_tut_list(request):
    try:
        per_page = int(request.GET['q'])
    except Exception as e:
        per_page = 10     # default value
    blog_list = request.user.bookmark_tutorials.all()
    paginator = Paginator(blog_list, per_page)
    page_number = request.GET.get("page")
    posts = paginator.get_page(page_number)
    return render(request, 'Dashboard/tutorials.html', {'posts': posts})




@login_required
def enquiry_list(request):
    try:
        per_page = int(request.GET['q'])
    except Exception as e:
        per_page = 10     # default value
    contact_list = Enquiry.objects.filter(
        Q(enqProduct__isnull=True) | Q(enqProduct__exact=''))
    paginator = Paginator(contact_list, per_page)  # Show 25 contacts per page.
    page_number = request.GET.get("page")
    page_obj = paginator.get_page(page_number)
    return render(request, 'cPanel/enqList.html', {'page_obj': page_obj})





@login_required
def enq(request, id):
    enq = Enquiry.objects.get(id=id)
    if enq.enqStatus == "Follow up":
        pass
    elif enq.enqStatus == "Done":
        pass
    else:
        enq.enqStatus = "Viewed"
        enq.save()

    if request.method == 'POST':
        if len(request.POST) == 2:
            enq.enqStatus = request.POST['status']
            enq.save()
        else:
            followup_assign(request, enq)
            return HttpResponseRedirect('/dash/enquiry/'+str(enq.id)+'/')
    return render(request, 'cPanel/enq.html', {'enq': enq})

@login_required
def enq_delete(request, id):
    enq = Enquiry.objects.get(id=id)
    if enq.enqProduct:
        rretObj=HttpResponseRedirect('/dash/product-enquiry/?page='+request.GET.get('page'))
    else:
        rretObj=HttpResponseRedirect('/dash/enquiry/?page='+request.GET.get('page'))
    flobj = FollowUp.objects.filter(enquiry=enq)
    for fl in flobj:
        fl.delete()
    enq.delete()
    return rretObj


@login_required
def users(request):
    if request.method == 'POST':
        id = request.POST['id']
        userobj = User.objects.get(id=id)
        if len(request.POST) ==2:
            if userobj.is_active == True:
                userobj.is_active = False
                msg="User "+userobj.username+" Deactivated Successfully"
            else:
                userobj.is_active = True
                msg="User "+userobj.username+" Activated Successfully"
                
            userobj.save()
            messages.success(request, msg)
            
            return HttpResponseRedirect('/dash/users/')
            
        if len(request.POST) ==5:
            if request.POST['password'] != request.POST['password2']:
                messages.warning(
                request, "Password & Re-Type Password did't match, Try again.")
            else:
                userobj.set_password(request.POST['password'])
                userobj.username=request.POST['username']
                userobj.save() 
                messages.success(request, "Password Updated Successfully for " + userobj.username)
            userobj.save()
            return HttpResponseRedirect('/dash/users/')
            
        if len(request.POST) ==3:
            userobj.delete()
            messages.success(request, "User "+userobj.username+" Deleted Successfully")
            
            return HttpResponseRedirect('/dash/users/')
            
    users = User.objects.filter(~Q(is_superuser=True))
    return render(request, 'cPanel/Users.html', {'users': users})

@login_required
def add_user(request):
    if request.method == 'POST':
        # print("Request=", request.POST)
        if User.objects.filter(username=request.POST['username']).exists():
            messages.warning(request, "Username "+request.POST['username']+" already exists")
            return HttpResponseRedirect('/dash/add-user/')

        if request.POST['password'] != request.POST['password2']:
            messages.warning(
                request, "Password & Re-Type Password did't match, Try again.")
            return HttpResponseRedirect('/dash/add-user/')

        userObj = User.objects.create(
            username=request.POST['username'],
            first_name=request.POST['name'],
            email=request.POST['email'],
            last_name=request.POST['contact'],
            is_staff=False,
            is_active=True
        )
        userObj.set_password(request.POST['password'])
        userObj.save()
        messages.success(request, "User Created Successfully")
        return HttpResponseRedirect('/dash/add-user/')
    else:
        return render(request, 'cPanel/NewUser.html')

@login_required
def change_password(request):
    if request.method == 'POST':
        if request.POST['new_password1'] != request.POST['new_password2']:
            messages.warning(request, "Password & Re-Type Password did't match, Try again.")
            return render(request, 'cPanel/change-password.html')
            
        else:
            userObj = request.user
            userObj.set_password(request.POST['new_password2'])
            userObj.save()
            logger.error(f'Profile Update: {e}')
            
            messages.success(request, "Password Updated Successfully")
            return HttpResponseRedirect('/dashboard/')

    else:
        return render(request, 'cPanel/change-password.html')
    # try:
    # except Exception as e:
    #         logger.error(f'Change Password: {e}')
        
@login_required
def profile(request):
    if request.method == 'POST':
        try:
            userObj = User.objects.get(username=request.user)
            userObj.first_name = request.POST['name']
            userObj.email = request.POST['email']
            userObj.last_name = request.POST['contact']
            userObj.save()
            logger.info(f'Profile Update: Done')
            
            messages.success(request, "Profile Updated Successfully")

            return HttpResponseRedirect('/dash/settings/profile/')
        except Exception as e:
            logger.error(f'Profile Update: {e}')
    else:
        return render(request, 'cPanel/profile.html')
